// 003 u_ibx
// 12 feb 2007

// -- (C) John COLIBRI 2007
// -- documentation: http://www.jcolibri.com

(*$r+*)

unit u_ibx;
  interface
    uses Classes, IbDatabase, IbSql, IbQuery, IbStoredProc
       ;

    function f_create_ib_database(p_database_name, p_user, p_password: String;
        p_dialect: Integer;
        p_c_ib_database: tIbDatabase): Boolean;
    procedure initialize_ib_database(p_database_name, p_user, p_password: String; p_c_ib_database: tIbDatabase);
    procedure display_ib_database_name_info(p_c_ib_database: tIbDatabase);
    function f_open_ib_database(p_message: String; p_c_ib_database: tIbDatabase): Boolean;
    procedure close_ib_database(p_message: String; p_c_ib_database: tIbDatabase);

    // function f_c_table_names(p_c_ib_database: tIbDatabase): tStringList;
    function f_c_table_name_list(p_c_ib_database: tIbDatabase): tStringList;
    function f_c_field_name_list(p_c_ib_database: tIbDatabase;
        p_table_name: String): tStringList;
    function f_c_field_info_ibquery(p_c_ib_database: tIbDatabase; p_table_name: String): tIbQuery;

    function f_c_view_name_list(p_c_ib_database: tIbDatabase): tStringList;

    function f_c_stored_procedure_list(p_c_ib_database: tIbDatabase): tStringList;
    function f_stored_procedure_script(p_c_ib_database: tIbDatabase; p_stored_procedure_name: String): String;
    function f_c_generator_list(p_c_ib_database: tIbDatabase): tStringList;
    function f_c_exception_list(p_c_ib_database: tIbDatabase): tStringList;
    function f_c_param_name_list(p_c_ib_stored_procedure: tIbStoredProc): tStringList;

    function f_execute_ibsql(p_c_ibqsl: tIbSql; p_request: String): Boolean;
    // procedure open_ibsql(p_c_ibsql: tIbSql; p_select_request: String);

    procedure open_ibquery(p_c_ibquery: tIbQuery; p_select_request: String);

    var g_c_ib_script_list: tStrings= Nil;

  implementation
    uses SysUtils
        , u_characters
        , u_file
        , u_c_display
        , u_types_constants, u_strings
        , IbExtract
        ;

    // -- database

    function f_create_ib_database(p_database_name, p_user, p_password: String;
        p_dialect: Integer;
        p_c_ib_database: tIbDatabase): Boolean;
      begin
        with p_c_ib_database, Params do
        begin
          Result:= False;
          Connected:= False;

          If FileExists(p_database_name)
            Then begin
                f_erase(p_database_name);
              end;

          DatabaseName:= p_database_name;
          SqlDialect:= p_dialect;
          // -- if not set, has "canceled at user's request"
          LoginPrompt:= False;
          Clear;

          Add('USER "'+ p_user+ '"');
          Add('PASSWORD "'+ p_password+ '"');
          Add('PAGE_SIZE 4096');

          Try
            CreateDatabase;
            Result:= True;
          except
            on e: Exception do
              display_bug_stop('*** err '+ e.Message);
          end; // try
        end; // with IbDatabase1, Params
      end; // f_create_ib_database

    procedure initialize_ib_database(p_database_name, p_user, p_password: String; p_c_ib_database: tIbDatabase);
        // -- set the name and parameters. Do not close, until has to change those
      begin
        with p_c_ib_database do
        begin
          // display(Params.Values['user_name']);
          if (DatabaseName= p_database_name)
              and (Params.Values['user_name']= p_user)
              and (Params.Values['password']= p_password)
            then Exit;

(*
          Params.Clear;
          Params.Add('user_name='+ p_user);
          Params.Add('password='+ p_password);
*)
          If Connected
            then Close;

          // -- if not set, has "canceled at user's request"
          LoginPrompt:= False;

          DatabaseName:= p_database_name;
          Params.Clear;
          Params.Add('user_name='+ p_user);
          Params.Add('password='+ p_password);
        end;
      end; // initialize_ib_database

    procedure display_ib_database_name_info(p_c_ib_database: tIbDatabase);
      begin
        with p_c_ib_database do
        begin
          display('name   >'+ DatabaseName+ '<');
          display('dia    >'+ IntToStr(SqlDialect));
          display('params >'+ Params.Text+ '|');
        end;
      end; // display_ib_database_name_info

    function f_open_ib_database(p_message: String; p_c_ib_database: tIbDatabase): Boolean;
      begin
        Result:= False;

        if p_message<> ''
          then display('> '+ p_message);

        with p_c_ib_database do
        begin
          Try
            if not Connected
              then // Open;
                   Connected:= True;
            Result:= True;
          Except
            on e: exception do
              display('ib_db_open_err '+ e.Message);
          end; // try except
        end; // with p_c_ib_database

        if p_message<> ''
          then display('< '+ p_message);
      end; // f_open_ib_database

    procedure close_ib_database(p_message: String; p_c_ib_database: tIbDatabase);
      begin
        if p_message<> ''
          then display('> '+ p_message);

        with p_c_ib_database do
        begin
          Try
            if Connected
              then Close;
          Except
            on e: exception do
              display('ib_db_close_err '+ e.Message);
          end; // try except
        end; // with p_c_ib_database

        if p_message<> ''
          then display('< '+ p_message);
      end; // close_ib_database

    // -- info

    function f_c_table_name_list(p_c_ib_database: tIbDatabase): tStringList;
      const k_select_table_names=
                  'SELECT RDB$RELATION_NAME'
                + '  FROM RDB$RELATIONS'
                + '  WHERE '
                + '      ('
                + '         (RDB$SYSTEM_FLAG = 0) '
                + '        OR'
                + '          (RDB$SYSTEM_FLAG IS NULL)'
                + '       )'
                + '     AND'
                + '      (RDB$VIEW_SOURCE IS NULL)'
                + '  ORDER BY RDB$RELATION_NAME'
                ;
      var l_c_ibquery: tIbQuery;
      begin
        f_open_ib_database('', p_c_ib_database);

        l_c_ibquery:= tIbQuery.Create(Nil);
        with l_c_ibquery do
        begin
          Database:= p_c_ib_database;
          open_ibquery(l_c_ibquery, k_select_table_names);

          Result:= tStringList.Create;
          While not Eof do
          begin
            Result.Add(Fields[0].AsString);
            Next;
          end; // While not Eof

          Close;
          Free;
        end; // with tIbQuery
      end; // f_c_table_name_list

    function f_c_field_name_list(p_c_ib_database: tIbDatabase;
        p_table_name: String): tStringList;
      const k_select_field_names=
                  'SELECT RDB$FIELD_NAME'
                + '  FROM RDB$RELATION_FIELDS'
                + '  WHERE RDB$RELATION_NAME= ''%s'' '
                ;
      var l_c_ibquery: tIbQuery;
      begin
        f_open_ib_database('', p_c_ib_database);

        l_c_ibquery:= tIbQuery.Create(Nil);
        with l_c_ibquery do
        begin
          Database:= p_c_ib_database;
          open_ibquery(l_c_ibquery, Format(k_select_field_names, [p_table_name]));

          Result:= tStringList.Create;
          While not Eof do
          begin
            Result.Add(Fields[0].AsString);
            Next;
          end; // While not Eof

          Close;
          Free;
        end; // with tIbQuery
      end; // f_c_field_name_list

    function f_c_field_info_ibquery(p_c_ib_database: tIbDatabase; p_table_name: String): tIbQuery;
      const k_select_field_info=
                  'select r.rdb$field_name, '
                + '    t.rdb$type_name, '
                + '    f.rdb$field_length, '
                + '    f.rdb$field_sub_type '
                + '  from rdb$relation_fields r, rdb$types t, rdb$fields f '
                + '  where r.rdb$relation_name= ''%s'' ' 
                + '    and f.rdb$field_name=r.rdb$field_source '
                + '    and t.rdb$field_name= ''RDB$FIELD_TYPE'' '
                + '    and f.rdb$field_type=t.rdb$type'
                ;
      var l_request: String;
      begin
        f_open_ib_database('', p_c_ib_database);

        Result:= tIbQuery.Create(Nil);
        with Result do
        begin
          Database:= p_c_ib_database;
          l_request:= Format(k_select_field_info, [p_table_name]);
          open_ibquery(Result, l_request);
        end; // with Result
      end; // f_c_field_info_ibquery

    function f_c_view_name_list(p_c_ib_database: tIbDatabase): tStringList;
      const k_select_view_names=
                 'SELECT RDB$RELATION_NAME '
               + '  FROM RDB$RELATIONS '
               + '  WHERE '
               + '      (RDB$SYSTEM_FLAG <> 1 OR RDB$SYSTEM_FLAG IS NULL) '
               + '    AND '
               + '      NOT RDB$VIEW_BLR IS NULL '
               + '    AND '
               + '      RDB$FLAGS = 1 '
               + '  ORDER BY RDB$RELATION_ID'
                ;
      var l_c_ibquery: tIbQuery;
      begin
        f_open_ib_database('', p_c_ib_database);

        l_c_ibquery:= tIbQuery.Create(Nil);
        with l_c_ibquery do
        begin
          Database:= p_c_ib_database;
          open_ibquery(l_c_ibquery, k_select_view_names);

          Result:= tStringList.Create;
          While not Eof do
          begin
            Result.Add(Fields[0].AsString);
            Next;
          end; // While not Eof

          Close;
          Free;
        end; // with tIbQuery
      end; // f_c_view_name_list

    function f_c_stored_procedure_list(p_c_ib_database: tIbDatabase): tStringList;
      const k_select_procedure_names=
                  'SELECT rdb$procedure_name'
                + '  FROM rdb$procedures'
                + '  ORDER BY rdb$procedure_name'
                ;
      var l_c_ibquery: tIbQuery;
      begin
        f_open_ib_database('', p_c_ib_database);

        l_c_ibquery:= tIbQuery.Create(Nil);
        with l_c_ibquery do
        begin
          Database:= p_c_ib_database;
          open_ibquery(l_c_ibquery, k_select_procedure_names);

          Result:= tStringList.Create;
          While not Eof do
          begin
            Result.Add(Fields[0].AsString);
            Next;
          end; // While not Eof

          Close;
          Free;
        end; // with tIbQuery
      end; // f_c_stored_procedure_list

    function f_stored_procedure_script(p_c_ib_database: tIbDatabase; p_stored_procedure_name: String): String;
      var l_c_ibextract: tIbExtract;
          l_item_index: Integer;
          l_the_line: String;
      begin
        f_open_ib_database('', p_c_ib_database);

        l_c_ibextract:= tIbExtract.Create(Nil);
        with l_c_ibextract do
        begin
          Database:= p_c_ib_database;
          ExtractObject(eoProcedure, p_stored_procedure_name);

          with Items do
          begin
            l_item_index:= 0;

            // -- extract everything between "ALTER ..." and " ^ "
            while l_item_index< Count do
            begin
              l_the_line:= Strings[l_item_index];
              if Pos('ALTER', l_the_line)> 0
                then break;
              Inc(l_item_index);
            end; // while l_item_index

            Result:= Trim(l_the_line)+ k_new_line;
            Result:= 'CREATE'+ f_remove_start(Result, 'ALTER');

            Inc(l_item_index);
            while l_item_index< Count do
            begin
              l_the_line:= Strings[l_item_index];
              if Trim(l_the_line)= '^'
                then break;
              Result:= Result+ l_the_line+ k_new_line;
              Inc(l_item_index);
            end; // while l_item_index
          end; // with Items

          Free;
        end; // while l_c_ibextract
      end; // f_stored_procedure_script

    function f_c_generator_list(p_c_ib_database: tIbDatabase): tStringList;
      const k_select_generator_names=
                'SELECT RDB$GENERATOR_NAME '
              + '  FROM RDB$GENERATORS '
              + '  WHERE (RDB$SYSTEM_FLAG IS NULL OR RDB$SYSTEM_FLAG <> 1) '
              + '  ORDER BY RDB$GENERATOR_NAME';
      var l_c_ibquery: tIbQuery;
      begin
        f_open_ib_database('', p_c_ib_database);

        l_c_ibquery:= tIbQuery.Create(Nil);
        with l_c_ibquery do
        begin
          Database:= p_c_ib_database;
          open_ibquery(l_c_ibquery, k_select_generator_names);

          Result:= tStringList.Create;
          While not Eof do
          begin
            Result.Add(Fields[0].AsString);
            Next;
          end; // While not Eof

          Close;
          Free;
        end; // with tIbQuery
      end; // f_c_generator_list

    function f_c_exception_list(p_c_ib_database: tIbDatabase): tStringList;
      const k_select_exception_names=
                'select * '
              + '  from RDB$EXCEPTIONS '
              + '  ORDER BY RDB$EXCEPTION_NAME';
      var l_c_ibquery: tIbQuery;
      begin
        f_open_ib_database('', p_c_ib_database);

        l_c_ibquery:= tIbQuery.Create(Nil);
        with l_c_ibquery do
        begin
          Database:= p_c_ib_database;
          open_ibquery(l_c_ibquery, k_select_exception_names);

          Result:= tStringList.Create;
          While not Eof do
          begin
            Result.Add(Fields[0].AsString);
            Next;
          end; // While not Eof

          Close;
          Free;
        end; // with tIbQuery
      end; // f_c_exception_list

    function f_c_param_name_list(p_c_ib_stored_procedure: tIbStoredProc): tStringList;
      var l_parameter_index: Integer;
      begin
        with p_c_ib_stored_procedure do
        begin
          f_open_ib_database('', Database);

          if not Prepared
            then Prepare;

          Result:= tStringList.Create;
          with Params do
            for l_parameter_index:= 0 to Count- 1 do
              Result.Add(Items[l_parameter_index].Name);
        end; // with p_c_ib_stored_procedure
      end; // f_c_param_name_list

    // -- ibsql

    function f_execute_ibsql(p_c_ibqsl: tIbSql; p_request: String): Boolean;
        // -- will close all (because commits)
      begin
        // display('> execute_query');
        Result:= False;

        with p_c_ibqsl do
        begin
          // display_strings_only(p_request);

          if True // m_exec_sql
            then
              try
                f_open_ib_database('', DataBase);

                Sql.Text:= p_request;

                if DataBase.DefaultTransaction.InTransaction
                  then DataBase.DefaultTransaction.Commit;
                DataBase.DefaultTransaction.StartTransaction;
                ExecQuery;
                DataBase.DefaultTransaction.Commit;

                // m_explain:= '  ok '+ p_title;
                // display(m_explain);
                if g_c_ib_script_list<> Nil
                  then g_c_ib_script_list.Add(p_request+ ';');

                Result:= True;
              except
                on e: Exception do
                  begin
                    // m_explain:= '*** pb_'+ p_title+ ' '+ e.Message;
                    display_bug_stop('*** pb_'+ e.Message);
                  end;
              end // try except
        end; // with m_c_ib_sql_ref
      end; // f_execute_ibsql

(*
    procedure open_ibsql(p_c_ibsql: tIbSql; p_select_request: String);
        // -- will close all (because commits)
      begin
        // display('> execute_query');
        Result:= False;

        with p_c_ibqsl do
        begin
          // display_strings_only(p_request);

          if True // m_exec_sql
            then
              try
                f_open_ib_database('', DataBase);

                Sql.Text:= p_request;

                if DataBase.DefaultTransaction.InTransaction
                  then DataBase.DefaultTransaction.Commit;
                DataBase.DefaultTransaction.StartTransaction;
                ExecQuery;
                DataBase.DefaultTransaction.Commit;

                // m_explain:= '  ok '+ p_title;
                // display(m_explain);
                if g_c_ib_script_list<> Nil
                  then g_c_ib_script_list.Add(p_request+ ';');

                Result:= True;
              except
                on e: Exception do
                  begin
                    // m_explain:= '*** pb_'+ p_title+ ' '+ e.Message;
                    display_bug_stop('*** pb_'+ e.Message);
                  end;
              end // try except
        end; // with m_c_ib_sql_ref
      end; // open_ibsql
*)

    // -- ibquery

    procedure open_ibquery(p_c_ibquery: tIbQuery; p_select_request: String);
      begin
        with p_c_ibquery, Sql do
        begin
          Close;
          Clear;
          Text:= p_select_request;
          try
            Open;
          except
            on e: exception do
              display('ibq_err '+ e.Message);
          end; // try except
        end; // open_ibquery
      end; // open_ibquery

    begin // u_zzz
    end. // u_zzz
